@@ -1,6 +1,6 @@
 /* library.c
  *
- * Copyright  2001 Lutz Mller
+ * Copyright 2001 Lutz Mueller
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -14,10 +14,12 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
+#define _BSD_SOURCE
+
 #include "config.h"
 
 #include <stdio.h>
@@ -734,6 +736,7 @@ camera_get_config (Camera* camera, CameraWidget** window, GPContext *context)
                                 gp_widget_add_choice (widget, name);
                 }
                 gp_widget_set_value (widget, _("None selected"));
+                gp_system_closedir (d);
         }
 
         /* TV output format */
@@ -865,6 +868,7 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
                 time_t xtime;
 
                 gp_widget_get_value (widget, &i);
+	        gp_widget_set_changed (widget, 0);
                 xtime = i;
                 tm_struct = localtime (&xtime);
                 date.year   = tm_struct->tm_year - 100;
@@ -880,6 +884,7 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
         gp_widget_get_child_by_label (section, _("Beep"), &widget);
         if (gp_widget_changed (widget)) {
                 gp_widget_get_value (widget, &c);
+	        gp_widget_set_changed (widget, 0);
                 if (strcmp (c, _("Off")) == 0)
                         beep = 0;
                 else
@@ -892,6 +897,7 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
         gp_widget_get_child_by_label (section, _("Self Timer Time"), &widget);
         if (gp_widget_changed (widget)) {
                 gp_widget_get_value (widget, &f);
+	        gp_widget_set_changed (widget, 0);
                 C(k_set_preference (camera->port, context,
                         K_PREFERENCE_SELF_TIMER_TIME, (int) f));
         }
@@ -900,6 +906,7 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
         gp_widget_get_child_by_label (section, _("Auto Off Time"), &widget);
         if (gp_widget_changed (widget)) {
                 gp_widget_get_value (widget, &f);
+	        gp_widget_set_changed (widget, 0);
                 C(k_set_preference (camera->port, context,
                                         K_PREFERENCE_AUTO_OFF_TIME, (int) f));
         }
@@ -909,6 +916,7 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
                                       &widget);
         if (gp_widget_changed (widget)) {
                 gp_widget_get_value (widget, &f);
+	        gp_widget_set_changed (widget, 0);
                 C(k_set_preference (camera->port, context,
                                 K_PREFERENCE_SLIDE_SHOW_INTERVAL, (int) f));
         }
@@ -917,6 +925,7 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
         gp_widget_get_child_by_label (section, _("Resolution"), &widget);
         if (gp_widget_changed (widget)) {
                 gp_widget_get_value (widget, &c);
+	        gp_widget_set_changed (widget, 0);
                 if (!strcmp (c, _("High (1152 x 872)")))
                         j = 1;
                 else if (!strcmp (c, _("Low (576 x 436)")))
@@ -936,6 +945,7 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
         C(gp_widget_get_child_by_label (section, _("Language"), &widget));
 	C(result = gp_widget_changed (widget));
 	if (result) {
+	        gp_widget_set_changed (widget, 0);
 		C(gp_widget_get_value (widget, &c));
                 if (strcmp (c, _("None selected"))) {
                         data = NULL;
@@ -961,6 +971,7 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
         /* TV Output Format */
         gp_widget_get_child_by_label (section, _("TV Output Format"), &widget);
         if (gp_widget_changed (widget)) {
+	        gp_widget_set_changed (widget, 0);
                 gp_widget_get_value (widget, &c);
                 if (strcmp (c, _("None selected"))) {
                         if (!strcmp (c, _("NTSC")))
@@ -979,6 +990,7 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
         /* Date Format */
         gp_widget_get_child_by_label (section, _("Date Format"), &widget);
         if (gp_widget_changed (widget)) {
+	        gp_widget_set_changed (widget, 0);
                 gp_widget_get_value (widget, &c);
                 if (strcmp (c, _("None selected"))) {
                         if (!strcmp (c, _("Month/Day/Year")))
@@ -1003,6 +1015,7 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
         /* Flash */
         gp_widget_get_child_by_label (section, _("Flash"), &widget);
         if (gp_widget_changed (widget)) {
+	        gp_widget_set_changed (widget, 0);
                 gp_widget_get_value (widget, &c);
                 if (!strcmp (c, _("Off")))
                         j = 0;
@@ -1021,6 +1034,7 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
         /* Exposure */
         gp_widget_get_child_by_label (section, _("Exposure"), &widget);
         if (gp_widget_changed (widget)) {
+	        gp_widget_set_changed (widget, 0);
                 gp_widget_get_value (widget, &f);
                 C(k_set_preference (camera->port, context,
                                         K_PREFERENCE_EXPOSURE, (int) f));
@@ -1039,6 +1053,8 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
                                       &widget_self_timer);
         if (gp_widget_changed (widget_focus) &&
             gp_widget_changed (widget_self_timer)) {
+	        gp_widget_set_changed (widget_focus, 0);
+	        gp_widget_set_changed (widget_self_timer, 0);
                 gp_widget_get_value (widget_focus, &c);
                 if (!strcmp (c, _("Auto")))
                         focus_self_timer = 2;
@@ -1065,8 +1081,6 @@ localization_file_read (Camera *camera, const char *file_name,
         int f;
         unsigned char c[] = "\0\0";
         unsigned long line_number;
-        unsigned char checksum;
-        unsigned long fcs;
         unsigned int d;
         char path[1024];
 
@@ -1133,9 +1147,10 @@ localization_file_read (Camera *camera, const char *file_name,
                         }
                         c[j] = (char) f;
                         if (j == 1) {
-                                if (sscanf (&c[0], "%X", &d) != 1) {
+                                if (sscanf ((char *)&c[0], "%X", &d) != 1) {
 					GP_DEBUG ("Error in localization "
 						  "file.");
+                                	fclose (file);
                                         return (GP_ERROR_CORRUPTED_DATA);
                                 }
                                 (*data)[*data_size] = d;
@@ -1144,7 +1159,7 @@ localization_file_read (Camera *camera, const char *file_name,
                                         gp_context_error (context,
 						_("Localization file too long!"));
                                         fclose (file);
-                                        return (FALSE);
+                                        return (GP_ERROR_CORRUPTED_DATA);
                                 }
                         }
                         j = 1 - j;
@@ -1154,13 +1169,11 @@ localization_file_read (Camera *camera, const char *file_name,
         fclose (file);
 
         /* Calculate and check checksum. */
-        checksum = 0;
 	gp_log (GP_LOG_DEBUG, "konica", "Checksum not implemented!");
         /*FIXME: There's a checksum at (*data)[100]. I could not
           figure out how it is calculated. */
 
         /* Calculate and check frame check sequence. */
-        fcs = 0;
 
 	gp_log (GP_LOG_DEBUG, "konica", "Frame check sequence "
 		"not implemented!");
