@@ -27,7 +27,6 @@
 #include <fcntl.h>
 #include <stdlib.h>
 #include <signal.h>
-#include <unistd.h>
 #include <stdbool.h>
 
 #ifndef LIBGPHOTO2
@@ -47,7 +46,7 @@
 
 #include "pslr.h"
 
-#define MAX_SEGMENTS 4
+#define MAX_SEGMENTS 20
 #define MAX_RESOLUTIONS 4
 #define POLL_INTERVAL 100000 /* Number of us to wait when polling */
 #define BLKSZ 65536 /* Block size for downloads; if too big, we get
@@ -80,6 +79,10 @@
         }                                                               \
     } while (0)
 
+#define PSLR_SUPPORTED_IMAGE_FORMAT_JPEG (1 << PSLR_IMAGE_FORMAT_JPEG)
+#define PSLR_SUPPORTED_IMAGE_FORMAT_RAW (1 << PSLR_IMAGE_FORMAT_RAW)
+#define PSLR_SUPPORTED_IMAGE_FORMAT_RAW_PLUS (1 << PSLR_IMAGE_FORMAT_RAW_PLUS)
+
 typedef enum {
     PSLR_BUFFER_SEGMENT_LAST = 2,
     PSLR_BUFFER_SEGMENT_INNER = 3,
@@ -104,6 +107,7 @@ typedef struct {
     uint32_t id2;
     const char *name;
     const char *resolution_steps[MAX_RESOLUTIONS];
+    int supported_formats;
 } ipslr_model_info_t;
 
 typedef struct {
@@ -159,22 +163,24 @@ static uint32_t get_uint32(uint8_t *buf);
 static bool is_k10d(ipslr_handle_t *p);
 static bool is_k20d(ipslr_handle_t *p);
 static bool is_k30(ipslr_handle_t *p);
+static bool is_k100ds(ipslr_handle_t *p);
 static bool is_istds(ipslr_handle_t *p);
 
 static pslr_progress_callback_t progress_callback = NULL;
 
 static ipslr_model_info_t camera_models[] = {
-    { PSLR_ID1_K30, PSLR_ID2_K30, "K30", { "16", "12", "8", "5" } },
-    { PSLR_ID1_K20D, PSLR_ID2_K20D, "K20D", { "14", "10", "6", "2" } },
-    { PSLR_ID1_K10D, PSLR_ID2_K10D, "K10D", { "10", "6", "2" } },
-    { PSLR_ID1_K110D, PSLR_ID2_K110D, "K110D", { "6", "4", "2" } },
-    { PSLR_ID1_K100D, PSLR_ID2_K100D, "K100D", { "6", "4", "2" } },
-    { PSLR_ID1_IST_DS2, PSLR_ID2_IST_DS2, "*ist DS2", { "6", "4", "2" } },
-    { PSLR_ID1_IST_DL, PSLR_ID2_IST_DL, "*ist DL", { "6", "4", "2" } },
-    { PSLR_ID1_IST_DS, PSLR_ID2_IST_DS, "*ist DS", { "6", "4", "2" } },
-    { PSLR_ID1_IST_D, PSLR_ID2_IST_D, "*ist D", { "6", "4", "2" } },
-    { PSLR_ID1_GX10, PSLR_ID2_GX10, "GX10", { "10", "6", "2" } },
-    { PSLR_ID1_GX20, PSLR_ID2_GX20, "GX20", { "14", "10", "6", "2" } },
+    { PSLR_ID1_K30, PSLR_ID2_K30, "K30", { "16", "12", "8", "5" }, PSLR_SUPPORTED_IMAGE_FORMAT_JPEG },
+    { PSLR_ID1_K20D, PSLR_ID2_K20D, "K20D", { "14", "10", "6", "2" }, PSLR_SUPPORTED_IMAGE_FORMAT_JPEG },
+    { PSLR_ID1_K10D, PSLR_ID2_K10D, "K10D", { "10", "6", "2" }, PSLR_SUPPORTED_IMAGE_FORMAT_JPEG },
+    { PSLR_ID1_K110D, PSLR_ID2_K110D, "K110D", { "6", "4", "2" }, PSLR_SUPPORTED_IMAGE_FORMAT_JPEG },
+    { PSLR_ID1_K100D, PSLR_ID2_K100D, "K100D", { "6", "4", "2" }, PSLR_SUPPORTED_IMAGE_FORMAT_JPEG },
+    { PSLR_ID1_K100DS, PSLR_ID2_K100DS, "K100DS", {}, PSLR_SUPPORTED_IMAGE_FORMAT_RAW },
+    { PSLR_ID1_IST_DS2, PSLR_ID2_IST_DS2, "*ist DS2", { "6", "4", "2" }, PSLR_SUPPORTED_IMAGE_FORMAT_JPEG },
+    { PSLR_ID1_IST_DL, PSLR_ID2_IST_DL, "*ist DL", { "6", "4", "2" }, PSLR_SUPPORTED_IMAGE_FORMAT_JPEG },
+    { PSLR_ID1_IST_DS, PSLR_ID2_IST_DS, "*ist DS", { "6", "4", "2" }, PSLR_SUPPORTED_IMAGE_FORMAT_JPEG },
+    { PSLR_ID1_IST_D, PSLR_ID2_IST_D, "*ist D", { "6", "4", "2" }, PSLR_SUPPORTED_IMAGE_FORMAT_JPEG },
+    { PSLR_ID1_GX10, PSLR_ID2_GX10, "GX10", { "10", "6", "2" }, PSLR_SUPPORTED_IMAGE_FORMAT_JPEG },
+    { PSLR_ID1_GX20, PSLR_ID2_GX20, "GX20", { "14", "10", "6", "2" }, PSLR_SUPPORTED_IMAGE_FORMAT_JPEG },
 };
 
 #ifndef LIBGPHOTO2
@@ -182,21 +188,21 @@ static ipslr_handle_t pslr;
 
 pslr_handle_t pslr_init()
 {
-    DIR *d;
+    gp_system_dir d;
     char nmbuf[256];
     char infobuf[64];
-    struct dirent *ent;
+    gp_system_dirent ent;
     int fd;
 
     memset(&pslr.devname, 0, sizeof(pslr.devname));
 
-    d = opendir("/sys/class/scsi_generic");
+    d = gp_system_opendir("/sys/class/scsi_generic");
 
     if (!d)
         return NULL;
 
     while (1) {
-        ent = readdir(d);
+        ent = gp_system_readdir(d);
         if (!ent)
             break;
         if (strcmp(ent->d_name, ".")==0 || strcmp(ent->d_name, "..") == 0)
@@ -236,7 +242,7 @@ pslr_handle_t pslr_init()
 
     }
 
-    closedir(d);
+    gp_system_closedir(d);
     if (pslr.devname[0] == '\0')
         return NULL;
 
@@ -268,7 +274,7 @@ int pslr_connect(pslr_handle_t h)
     CHECK(ipslr_identify(p));
     CHECK(ipslr_status_full(p, &p->status));
     DPRINT("init bufmask=0x%x\n", p->status.bufmask);
-    if (is_k10d(p) || is_k20d(p) || is_k30(p))
+    if (is_k10d(p) || is_k20d(p) || is_k30(p) || is_k100ds(p))
         CHECK(ipslr_cmd_00_09(p, 2));
     CHECK(ipslr_status_full(p, &p->status));
     CHECK(ipslr_cmd_10_0a(p, 1));
@@ -482,6 +488,13 @@ int pslr_set_image_format(pslr_handle_t h, pslr_image_format_t format)
     return PSLR_OK;
 }
 
+int pslr_is_image_format_supported(pslr_handle_t h, pslr_image_format_t format)
+{
+    ipslr_handle_t *p = (ipslr_handle_t *) h;
+    if (format < 0 || format >= PSLR_IMAGE_FORMAT_MAX || !p->model)
+        return false;
+    return (1 << format) & p->model->supported_formats;
+}
 
 int pslr_set_raw_format(pslr_handle_t h, pslr_raw_format_t format)
 {
@@ -576,7 +589,7 @@ int pslr_buffer_open(pslr_handle_t h, int bufno, int buftype, int bufres)
             CHECK(ipslr_buffer_segment_info(p, &info));
             CHECK(ipslr_next_segment(p));
             DPRINT("Recover: type=%d\n", info.type);
-        } while (++retry2 < 10 && info.type != PSLR_BUFFER_SEGMENT_LAST);
+        } while (++retry2 < 100 && info.type != PSLR_BUFFER_SEGMENT_LAST);
     }
 
     if (retry == 3)
@@ -603,7 +616,7 @@ int pslr_buffer_open(pslr_handle_t h, int bufno, int buftype, int bufres)
         CHECK(ipslr_next_segment(p));
         buf_total += info.length;
         i++;
-    } while (i < 9 && info.type != PSLR_BUFFER_SEGMENT_LAST);
+    } while (i < 100 && info.type != PSLR_BUFFER_SEGMENT_LAST);
     p->segment_count = j;
     p->buffer_len = buf_total;
     p->offset = 0;
@@ -957,6 +970,23 @@ static int ipslr_status_full(ipslr_handle_t *p, pslr_status *status)
         return PSLR_OK;
     }
 
+    if (p->model && is_k100ds(p)) {
+        /* K100DS (Super) status block */
+        if (n != 264)  {
+            DPRINT("only got %d bytes\n", n);
+            return PSLR_READ_ERROR;
+        }
+
+        CHECK(read_result(p, buf, n));
+        memset(status, 0, sizeof(*status));
+
+        status->bufmask = get_uint32(&buf[0x10]);
+        status->image_format = PSLR_IMAGE_FORMAT_RAW;
+        status->raw_format = PSLR_RAW_FORMAT_PEF;
+
+        return PSLR_OK;
+    }
+
     if (p->model && is_istds(p)) {
         /* *ist DS status block */
         if (n != 0x108) {
@@ -991,7 +1021,7 @@ static int ipslr_press_shutter(ipslr_handle_t *p)
     CHECK(ipslr_write_args(p, 1, 2));
     CHECK(command(p, 0x10, 0x05, 0x04));
     get_status(p);
-    DPRINT("shutter result code: 0x%x\n", r);
+    DPRINT("shutter result code: 0x%x\n", p->status);
     return PSLR_OK;
 }
 
@@ -1051,7 +1081,7 @@ static int ipslr_next_segment(ipslr_handle_t *p)
     int r;
     CHECK(ipslr_write_args(p, 1, 0));
     CHECK(command(p, 0x04, 0x01, 0x04));
-    usleep(100000); /* needed !! 100 too short, 1000 not short enough for PEF */
+    usleep(10000); /* needed !! 100 too short, 1000 not short enough for PEF */
     r = get_status(p);
     if (r == 0)
         return PSLR_OK;
@@ -1459,6 +1489,15 @@ static bool is_k30(ipslr_handle_t *p)
     return false;
 }
 
+static bool is_k100ds(ipslr_handle_t *p)
+{
+    if (p->model && p->model->id1 == PSLR_ID1_K100DS
+        && p->model->id2 == PSLR_ID2_K100DS)
+        return true;
+
+    return false;
+}
+
 static bool is_istds(ipslr_handle_t *p)
 {
     if (p->model && p->model->id1 == PSLR_ID1_IST_DS 
