@@ -28,7 +28,7 @@
  *
  * Nikon WU-1* adapters might use 0011223344556677 as GUID always...
  */
-#define _BSD_SOURCE
+#define _DEFAULT_SOURCE
 #include "config.h"
 
 #include <stdlib.h>
@@ -101,17 +101,33 @@ static uint16_t ptp_ptpip_event (PTPParams* params, PTPContainer* event, int wai
 
 /* send / receive functions */
 uint16_t
-ptp_ptpip_sendreq (PTPParams* params, PTPContainer* req)
+ptp_ptpip_sendreq (PTPParams* params, PTPContainer* req, int dataphase)
 {
 	int		ret;
 	int		len = 18+req->Nparam*4;
 	unsigned char 	*request = malloc(len);
 
+	switch (req->Nparam) {
+	default:
+	case 0: GP_LOG_D ("Sending PTP_OC 0x%0x (%s) request...", req->Code, ptp_get_opcode_name(params, req->Code));
+		break;
+	case 1: GP_LOG_D ("Sending PTP_OC 0x%0x (%s) (0x%x) request...", req->Code, ptp_get_opcode_name(params, req->Code), req->Param1);
+		break;
+	case 2: GP_LOG_D ("Sending PTP_OC 0x%0x (%s) (0x%x,0x%x) request...", req->Code, ptp_get_opcode_name(params, req->Code), req->Param1, req->Param2);
+		break;
+	case 3: GP_LOG_D ("Sending PTP_OC 0x%0x (%s) (0x%x,0x%x,0x%x) request...", req->Code, ptp_get_opcode_name(params, req->Code), req->Param1, req->Param2, req->Param3);
+		break;
+	}
+
 	ptp_ptpip_check_event (params);
 
 	htod32a(&request[ptpip_type],PTPIP_CMD_REQUEST);
 	htod32a(&request[ptpip_len],len);
-	htod32a(&request[ptpip_cmd_dataphase],1);	/* FIXME: dataphase handling */
+	/* sending data = 2, receiving data or no data = 1 */
+	if ((dataphase&PTP_DP_DATA_MASK) == PTP_DP_SENDDATA)
+		htod32a(&request[ptpip_cmd_dataphase],2);
+	else
+		htod32a(&request[ptpip_cmd_dataphase],1);
 	htod16a(&request[ptpip_cmd_code],req->Code);
 	htod32a(&request[ptpip_cmd_transid],req->Transaction_ID);
 
@@ -125,7 +141,7 @@ ptp_ptpip_sendreq (PTPParams* params, PTPContainer* req)
 	default:
 		break;
 	}
-	gp_log_data ( "ptpip/oprequest", (char*)request, len);
+	GP_LOG_DATA ( (char*)request, len, "ptpip/oprequest data:");
 	ret = write (params->cmdfd, request, len);
 	free (request);
 	if (ret == -1)
@@ -149,7 +165,7 @@ ptp_ptpip_generic_read (PTPParams *params, int fd, PTPIPHeader *hdr, unsigned ch
 			perror ("read PTPIPHeader");
 			return PTP_RC_GeneralError;
 		}
-		gp_log_data ( "ptpip/generic_read", (char*)xhdr+curread, ret);
+		GP_LOG_DATA ((char*)xhdr+curread, ret, "ptpip/generic_read header:");
 		curread += ret;
 		if (ret == 0) {
 			GP_LOG_E ("End of stream after reading %d bytes of ptpipheader", ret);
@@ -174,7 +190,7 @@ ptp_ptpip_generic_read (PTPParams *params, int fd, PTPIPHeader *hdr, unsigned ch
 			free (*data);*data = NULL;
 			return PTP_RC_GeneralError;
 		} else {
-			gp_log_data ( "ptpip/generic_read", (char*)((*data)+curread), ret);
+			GP_LOG_DATA ((char*)((*data)+curread), ret, "ptpip/generic_read data:");
 		}
 		if (ret == 0)
 			break;
@@ -229,12 +245,13 @@ ptp_ptpip_senddata (PTPParams* params, PTPContainer* ptp,
 	int		ret;
 	unsigned char*	xdata;
 
+	GP_LOG_D ("Sending PTP_OC 0x%0x (%s) data...", ptp->Code, ptp_get_opcode_name(params, ptp->Code));
 	htod32a(&request[ptpip_type],PTPIP_START_DATA_PACKET);
 	htod32a(&request[ptpip_len],sizeof(request));
 	htod32a(&request[ptpip_startdata_transid  + 8],ptp->Transaction_ID);
 	htod32a(&request[ptpip_startdata_totallen + 8],size);
 	htod32a(&request[ptpip_startdata_unknown  + 8],0);
-	gp_log_data ( "ptpip/senddata", (char*)request, sizeof(request));
+	GP_LOG_DATA ((char*)request, sizeof(request), "ptpip/senddata header:");
 	ret = write (params->cmdfd, request, sizeof(request));
 	if (ret == -1)
 		perror ("sendreq/write to cmdfd");
@@ -267,7 +284,7 @@ ptp_ptpip_senddata (PTPParams* params, PTPContainer* ptp,
 		htod32a(&xdata[ptpip_type], type);
 		htod32a(&xdata[ptpip_len], towrite2);
 		htod32a(&xdata[ptpip_data_transid+8], ptp->Transaction_ID);
-		gp_log_data("ptpip/senddata", (char*)xdata, towrite2);
+		GP_LOG_DATA ((char*)xdata, towrite2, "ptpip/senddata data:");
 		written = 0;
 		while (written < towrite2) {
 			ret = write (params->cmdfd, xdata+written, towrite2-written);
@@ -292,13 +309,14 @@ ptp_ptpip_getdata (PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler
 	unsigned long		toread, curread;
 	int			xret;
 
+	GP_LOG_D ("Reading PTP_OC 0x%0x (%s) data...", ptp->Code, ptp_get_opcode_name(params, ptp->Code));
 	ret = ptp_ptpip_cmd_read (params, &hdr, &xdata);
 	if (ret != PTP_RC_OK)
 		return ret;
 
 	if (dtoh32(hdr.type) == PTPIP_CMD_RESPONSE) { /* might happen if we have no data transfer due to error? */
-		GP_LOG_E ("Unexpected ptp response, code %x", dtoh32a(&xdata[8]));
-		return PTP_RC_GeneralError;
+		GP_LOG_E ("Unexpected ptp response, ptp code %x", dtoh16a(&xdata[0]));
+		return dtoh16a(&xdata[0]);
 	}
 	if (dtoh32(hdr.type) != PTPIP_START_DATA_PACKET) {
 		GP_LOG_E ("got reply type %d\n", dtoh32(hdr.type));
@@ -372,22 +390,38 @@ ptp_ptpip_getresp (PTPParams* params, PTPContainer* resp)
 	uint16_t 	ret;
 	int		n;
 
+	GP_LOG_D ("Reading PTP_OC 0x%0x (%s) response...", resp->Code, ptp_get_opcode_name(params, resp->Code));
+retry:
 	ret = ptp_ptpip_cmd_read (params, &hdr, &data);
 	if (ret != PTP_RC_OK)
 		return ret;
 
-	resp->Code		= dtoh16a(&data[ptpip_resp_code]);
-	resp->Transaction_ID	= dtoh32a(&data[ptpip_resp_transid]);
-	n = (dtoh32(hdr.length) - sizeof(hdr) - ptpip_resp_param1)/sizeof(uint32_t);
-	switch (n) {
-	case 5: resp->Param5 = dtoh32a(&data[ptpip_resp_param5]);
-	case 4: resp->Param4 = dtoh32a(&data[ptpip_resp_param4]);
-	case 3: resp->Param3 = dtoh32a(&data[ptpip_resp_param3]);
-	case 2: resp->Param2 = dtoh32a(&data[ptpip_resp_param2]);
-	case 1: resp->Param1 = dtoh32a(&data[ptpip_resp_param1]);
-	case 0: break;
+	switch (dtoh32(hdr.type)) {
+	case PTPIP_END_DATA_PACKET:
+		GP_LOG_D("PTPIP_END_DATA_PACKET");
+		resp->Transaction_ID	= dtoh32a(&data[0]);
+		free (data);
+		data = NULL;
+		goto retry;
+	case PTPIP_CMD_RESPONSE:
+		GP_LOG_D("PTPIP_CMD_RESPONSE");
+		resp->Code		= dtoh16a(&data[ptpip_resp_code]);
+		resp->Transaction_ID	= dtoh32a(&data[ptpip_resp_transid]);
+		n = (dtoh32(hdr.length) - sizeof(hdr) - ptpip_resp_param1)/sizeof(uint32_t);
+		switch (n) {
+		case 5: resp->Param5 = dtoh32a(&data[ptpip_resp_param5]);
+		case 4: resp->Param4 = dtoh32a(&data[ptpip_resp_param4]);
+		case 3: resp->Param3 = dtoh32a(&data[ptpip_resp_param3]);
+		case 2: resp->Param2 = dtoh32a(&data[ptpip_resp_param2]);
+		case 1: resp->Param1 = dtoh32a(&data[ptpip_resp_param1]);
+		case 0: break;
+		default:
+			GP_LOG_E ("response got %d parameters?", n);
+			break;
+		}
+		break;
 	default:
-		GP_LOG_E ("response got %d parameters?", n);
+		GP_LOG_E ("response type %d packet?", dtoh32(hdr.type));
 		break;
 	}
 	free (data);
@@ -407,8 +441,12 @@ ptp_ptpip_init_command_request (PTPParams* params)
 	unsigned char	guid[16];
 	
 	ptp_nikon_getptpipguid(guid);
+#if !defined (WIN32)
 	if (gethostname (hostname, sizeof(hostname)))
 		return PTP_RC_GeneralError;
+#else
+	strcpy (hostname, "gpwindows");
+#endif
 	len = ptpip_initcmd_name + (strlen(hostname)+1)*2 + 4;
 
 	cmdrequest = malloc(len);
@@ -424,7 +462,7 @@ ptp_ptpip_init_command_request (PTPParams* params)
 	htod16a(&cmdrequest[ptpip_initcmd_name+(strlen(hostname)+1)*2],PTPIP_VERSION_MINOR);
 	htod16a(&cmdrequest[ptpip_initcmd_name+(strlen(hostname)+1)*2+2],PTPIP_VERSION_MAJOR);
 
-	gp_log_data ( "ptpip/init_cmd", (char*)cmdrequest, len);
+	GP_LOG_DATA ((char*)cmdrequest, len, "ptpip/init_cmd data:");
 	ret = write (params->cmdfd, cmdrequest, len);
 	free (cmdrequest);
 	if (ret == -1) {
@@ -485,7 +523,7 @@ ptp_ptpip_init_event_request (PTPParams* params)
 	htod32a(&evtrequest[ptpip_len],ptpip_eventinit_size);
 	htod32a(&evtrequest[ptpip_eventinit_idx],params->eventpipeid);
 
-	gp_log_data ( "ptpip/init_event", (char*)evtrequest, ptpip_eventinit_size);
+	GP_LOG_DATA ((char*)evtrequest, ptpip_eventinit_size, "ptpip/init_event data:");
 	ret = write (params->evtfd, evtrequest, ptpip_eventinit_size);
 	if (ret == -1) {
 		perror("write init evt request");
@@ -665,7 +703,7 @@ ptp_nikon_getptpipguid (unsigned char* guid) {
 int
 ptp_ptpip_connect (PTPParams* params, const char *address) {
 	char 		*addr, *s, *p;
-	int		port;
+	int		port, eventport, tries;
 	struct sockaddr_in	saddr;
 	uint16_t	ret;
 
@@ -685,7 +723,7 @@ ptp_ptpip_connect (PTPParams* params, const char *address) {
 	}
 	*s = '\0';
 	p = strchr (s+1,':');
-	port = 15740;
+	eventport = port = 15740;
 	if (p) {
 		*p = '\0';
 		if (!sscanf (p+1,"%d",&port)) {
@@ -693,6 +731,15 @@ ptp_ptpip_connect (PTPParams* params, const char *address) {
 			free (addr);
 			return GP_ERROR_BAD_PARAMETERS;
 		}
+		/* different event port ? */
+		p = strchr (p+1,':');
+		if (p) {
+			if (!sscanf (p+1,"%d",&eventport)) {
+				fprintf(stderr,"failed to scan for eventport in %s\n", p+1);
+				free (addr);
+				return GP_ERROR_BAD_PARAMETERS;
+			}
+		}
 	}
 	if (!inet_aton (s+1,  &saddr.sin_addr)) {
 		fprintf(stderr,"failed to scan for addr in %s\n", s+1);
@@ -731,16 +778,25 @@ ptp_ptpip_connect (PTPParams* params, const char *address) {
 		close (params->evtfd);
 		return translate_ptp_result (ret);
 	}
-	if (-1 == connect (params->evtfd, (struct sockaddr*)&saddr, sizeof(struct sockaddr_in))) {
+	/* seen on Ricoh Theta, camera is not immediately ready. try again two times. */
+	tries = 2;
+	saddr.sin_port		= htons(eventport);
+	do {
+		if (-1 != connect (params->evtfd, (struct sockaddr*)&saddr, sizeof(struct sockaddr_in)))
+			break;
+		if ((errno == ECONNREFUSED) && (tries--)) {
+			GP_LOG_D ("event connect failed, retrying after short wait");
+			usleep(100*1000);
+			continue;
+		}
 		GP_LOG_E ("could not connect event");
 		close (params->cmdfd);
 		close (params->evtfd);
 		return GP_ERROR_IO;
-	}
+	} while (1);
 	ret = ptp_ptpip_init_event_request (params);
-	if (ret != PTP_RC_OK) {
+	if (ret != PTP_RC_OK)
 		return translate_ptp_result (ret);
-	}
 	ret = ptp_ptpip_init_event_ack (params);
 	if (ret != PTP_RC_OK)
 		return translate_ptp_result (ret);
