@@ -1,7 +1,7 @@
 /* usb.c
  *
  * Copyright (C) 2001-2004 Mariusz Woloszyn <emsi@ipartners.pl>
- * Copyright (C) 2003-2014 Marcus Meissner <marcus@jet.franken.de>
+ * Copyright (C) 2003-2016 Marcus Meissner <marcus@jet.franken.de>
  * Copyright (C) 2006-2007 Linus Walleij <triad@df.lth.se>
  *
  * This library is free software; you can redistribute it and/or
@@ -20,7 +20,7 @@
  * Boston, MA  02110-1301  USA
  */
 
-#define _BSD_SOURCE
+#define _DEFAULT_SOURCE
 #include <config.h>
 #include "ptp.h"
 #include "ptp-private.h"
@@ -30,7 +30,6 @@
 #include <stdarg.h>
 #include <stdio.h>
 #include <string.h>
-#include <unistd.h>
 
 #include <gphoto2/gphoto2-library.h>
 #include <gphoto2/gphoto2-port-log.h>
@@ -61,7 +60,8 @@
 
 /* PTP2_FAST_TIMEOUT: how long (in milliseconds) we should wait for
  * an URB to come back on an interrupt endpoint */
-#define PTP2_FAST_TIMEOUT       100
+/* 100 is not enough for various cameras types. 150 seems to work better */
+#define PTP2_FAST_TIMEOUT       150
 
 /* Pack / unpack functions */
 
@@ -70,13 +70,24 @@
 /* send / receive functions */
 
 uint16_t
-ptp_usb_sendreq (PTPParams* params, PTPContainer* req)
+ptp_usb_sendreq (PTPParams* params, PTPContainer* req, int dataphase)
 {
 	int res, towrite, do_retry = TRUE;
 	PTPUSBBulkContainer usbreq;
 	Camera *camera = ((PTPData *)params->data)->camera;
 
-	GP_LOG_D ("Sending PTP_OC 0x%0x request...", req->Code);
+	switch (req->Nparam) {
+	default:
+	case 0: GP_LOG_D ("Sending PTP_OC 0x%0x (%s) request...", req->Code, ptp_get_opcode_name(params, req->Code));
+		break;
+	case 1: GP_LOG_D ("Sending PTP_OC 0x%0x (%s) (0x%x) request...", req->Code, ptp_get_opcode_name(params, req->Code), req->Param1);
+		break;
+	case 2: GP_LOG_D ("Sending PTP_OC 0x%0x (%s) (0x%x,0x%x) request...", req->Code, ptp_get_opcode_name(params, req->Code), req->Param1, req->Param2);
+		break;
+	case 3: GP_LOG_D ("Sending PTP_OC 0x%0x (%s) (0x%x,0x%x,0x%x) request...", req->Code, ptp_get_opcode_name(params, req->Code), req->Param1, req->Param2, req->Param3);
+		break;
+	}
+
 	/* build appropriate USB container */
 	usbreq.length=htod32(PTP_USB_BULK_REQ_LEN-
 		(sizeof(uint32_t)*(5-req->Nparam)));
@@ -122,7 +133,7 @@ ptp_usb_senddata (PTPParams* params, PTPContainer* ptp,
 	int usecontext = (size > CONTEXT_BLOCK_SIZE);
 	GPContext *context = ((PTPData *)params->data)->context;
 
-	GP_LOG_D ("Sending PTP_OC 0x%0x data...", ptp->Code);
+	GP_LOG_D ("Sending PTP_OC 0x%0x (%s) data...", ptp->Code, ptp_get_opcode_name(params, ptp->Code));
 	/* build appropriate USB container */
 	usbdata.length	= htod32(PTP_USB_BULK_HDR_LEN+size);
 	usbdata.type	= htod16(PTP_USB_CONTAINER_DATA);
@@ -203,7 +214,7 @@ finalize:
 }
 
 static uint16_t
-ptp_usb_getpacket(PTPParams *params, PTPUSBBulkContainer *packet, uint32_t *rlen)
+ptp_usb_getpacket(PTPParams *params, PTPUSBBulkContainer *packet, uint32_t maxsize, uint32_t *rlen)
 {
 	int		tries = 0, result;
 	Camera		*camera = ((PTPData *)params->data)->camera;
@@ -212,6 +223,10 @@ ptp_usb_getpacket(PTPParams *params, PTPUSBBulkContainer *packet, uint32_t *rlen
 	if (params->response_packet_size > 0) {
 		GP_LOG_D ("Returning previously buffered response packet.");
 		/* If there is a buffered packet, just use it. */
+
+		if (params->response_packet_size > maxsize)
+			params->response_packet_size = maxsize;
+
 		memcpy(packet, params->response_packet, params->response_packet_size);
 		*rlen = params->response_packet_size;
 		free(params->response_packet);
@@ -240,7 +255,7 @@ retry:
 	return PTP_ERROR_IO;
 }
 
-#define READLEN 64*1024 /* read blob size */
+#define READLEN 512*1024 /* read blob size, mostly to avoid reading all of it at once. */
 
 uint16_t
 ptp_usb_getdata (PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler)
@@ -253,16 +268,34 @@ ptp_usb_getdata (PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler)
 	int		report_progress, progress_id = 0, do_retry = TRUE, res = GP_OK;
 	GPContext *context = ((PTPData *)params->data)->context;
 
-	GP_LOG_D ("Reading PTP_OC 0x%0x data...", ptp->Code);
+	GP_LOG_D ("Reading PTP_OC 0x%0x (%s) data...", ptp->Code, ptp_get_opcode_name(params, ptp->Code));
 	PTP_CNT_INIT(usbdata);
 
-	ret = ptp_usb_getpacket (params, &usbdata, &bytes_read);
+	ret = ptp_usb_getpacket (params, &usbdata, sizeof(usbdata), &bytes_read);
 	if (ret != PTP_RC_OK)
 		goto exit;
+
+	if (bytes_read < PTP_USB_BULK_HDR_LEN) {
+		GP_LOG_E ("Read short bulk packet in data phase %d vs %d vs min len %d", bytes_read, dtoh32(usbdata.length), (uint32_t)PTP_USB_BULK_HDR_LEN);
+		ret = PTP_ERROR_IO;
+		goto exit;
+	}
+
 	if (dtoh16(usbdata.type) != PTP_USB_CONTAINER_DATA) {
 		/* We might have got a response instead. On error for instance. */
-		/* TODO: check if bytes_read == usbdata.length */
 		if (dtoh16(usbdata.type) == PTP_USB_CONTAINER_RESPONSE) {
+			/* responses are short and we should have it as-is right now */
+			if (bytes_read != dtoh32(usbdata.length)) {
+				GP_LOG_E ("Read broken ptp response in data phase, read %d vs %d", bytes_read, dtoh32(usbdata.length));
+				ret = PTP_ERROR_IO;
+				goto exit;
+			}
+			if (dtoh32(usbdata.length) > sizeof(usbdata)) {
+				GP_LOG_E ("Read too large ptp response in data phase, packet %d bytes large", dtoh32(usbdata.length));
+				ret = PTP_ERROR_IO;
+				goto exit;
+			}
+			/* FIXME: maximum size of response packet perhaps ? */
 			params->response_packet = malloc(dtoh32(usbdata.length));
 			if (!params->response_packet) return PTP_RC_GeneralError;
 			memcpy(params->response_packet, (uint8_t *) &usbdata, dtoh32(usbdata.length));
@@ -408,14 +441,20 @@ ptp_usb_getresp (PTPParams* params, PTPContainer* resp)
 	PTPUSBBulkContainer	usbresp;
 	/*GPContext		*context = ((PTPData *)params->data)->context;*/
 
-	GP_LOG_D ("Reading PTP_OC 0x%0x response...", resp->Code);
+	GP_LOG_D ("Reading PTP_OC 0x%0x (%s) response...", resp->Code, ptp_get_opcode_name(params, resp->Code));
 	PTP_CNT_INIT(usbresp);
 	/* read response, it should never be longer than sizeof(usbresp) */
-	ret = ptp_usb_getpacket(params, &usbresp, &rlen);
+	ret = ptp_usb_getpacket(params, &usbresp, sizeof(usbresp), &rlen);
 
 	if (ret!=PTP_RC_OK) {
 		ret = PTP_ERROR_IO;
 	} else
+	if (rlen < 12) {
+		ret = PTP_ERROR_IO;
+	} else
+	if (rlen != dtoh32(usbresp.length)) {
+		ret = PTP_ERROR_IO;
+	} else
 	if (dtoh16(usbresp.type)!=PTP_USB_CONTAINER_RESPONSE) {
 		ret = PTP_ERROR_RESP_EXPECTED;
 	} else
