@@ -1,6 +1,6 @@
 /* Sony DSC-F55 & MSAC-SR1 - gPhoto2 camera library
- * Copyright  2001, 2002, 2004 Raymond Penners <raymond@dotsphinx.com>
- * Copyright  2000 Mark Davies <mdavies@dial.pipex.com>
+ * Copyright 2001, 2002, 2004 Raymond Penners <raymond@dotsphinx.com>
+ * Copyright 2000 Mark Davies <mdavies@dial.pipex.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -14,10 +14,12 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
+#define _BSD_SOURCE
+
 #include "config.h"
 
 #include <stdio.h>
@@ -155,7 +157,7 @@ sony_baud_to_id(long baud)
 static int
 sony_read_byte(Camera * camera, unsigned char *b)
 {
-	int n = gp_port_read(camera->port, b, 1);
+	int n = gp_port_read(camera->port, (char *)b, 1);
 	if (n != 1)
 		return GP_ERROR;
 	else
@@ -314,7 +316,7 @@ sony_packet_write(Camera * camera, Packet * p)
 	 **/
 	usleep(10000);
 
-	rc = gp_port_write(camera->port, &START_PACKET, 1);
+	rc = gp_port_write(camera->port, (char *)&START_PACKET, 1);
 
 	p->buffer[p->length] = p->checksum;
 
