@@ -24,8 +24,9 @@
    
   \par
   You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  along with this program; if not, write to the 
+  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+  Boston, MA  02110-1301  USA
 */
 #include "exif.h"
 
@@ -142,15 +143,15 @@ static const struct tagarray{
  */
 static int gpi_getvalue(unsigned char *data,int tagind);
 static int gpi_datsize(unsigned char *data,int tagind);
-static int gpi_tagnum( char *data,int tagind);
+static int gpi_tagnum( unsigned char *data,int tagind);
 static int gpi_getintval(unsigned char *data, int tagnum);
 
-static long gpi_exif_get_lilend(char *data, int size);
-static int gpe_theval(char *data,int tagind);
-static void gpi_setval(char *data,int tagind,long newval);
+static long gpi_exif_get_lilend(unsigned char *data, int size);
+static int gpe_theval(unsigned char *data,int tagind);
+static void gpi_setval(unsigned char *data,int tagind,long newval);
 
 static long exif_next_ifd(unsigned char *exif,int num);
-static int gpi_exif_get_comment(exifparser *exifdat, char *comment);
+static int gpi_exif_get_comment(exifparser *exifdat, char **comment);
 static int gpi_exif_get_field( int tag_number, int ifd, exifparser *exifdata, ExifData *tag_data);
 /*
  *  Utility functions: get fields in little-endian format,
@@ -165,7 +166,7 @@ static int gpi_exif_get_field( int tag_number, int ifd, exifparser *exifdata, Ex
  *
  * Returns : the value
  */
-static long gpi_exif_get_lilend(char *data, int size){
+static long gpi_exif_get_lilend(unsigned char *data, int size){
   long total;
 
   total=0;
@@ -180,7 +181,7 @@ static long gpi_exif_get_lilend(char *data, int size){
 /*
  *  Return "value" of directory entry at tagind
  */
-static int gpe_theval(char *data,int tagind){
+static int gpe_theval(unsigned char *data,int tagind){
  return(gpi_exif_get_lilend(data+tagind*12+10,4));
 }
 
@@ -188,7 +189,7 @@ static int gpe_theval(char *data,int tagind){
 /*
  *  Set the "value" of directory entry at tagind.
  */
-static void gpi_setval(char *data,int tagind,long newval){
+static void gpi_setval(unsigned char *data,int tagind,long newval){
   int i;
   for (i=0;i<4;i++) data[tagind*12+10+i]=0xff&(newval>>(i*8));
   if (gpi_getvalue(data,tagind)!=newval) 
@@ -234,8 +235,8 @@ static int exif_parse_data(exifparser *exifdat) {
 
   /* First, verify that we really have an EXIF file */
   /* Note: maybe we could also generalize to TIFF... */
-  if ((strncmp("Exif",exifdat->header+6,4)) ||
-      strncmp("\377\330\377\341",exifdat->header,4)){
+  if ((memcmp("Exif",exifdat->header+6,4)) ||
+      memcmp("\377\330\377\341",exifdat->header,4)){
     fprintf(stderr,"Not exif data\n");
     return(-1);
   }
@@ -299,7 +300,7 @@ static int exif_parse_data(exifparser *exifdat) {
  */
 static int gpi_exif_get_field( int tag_number, int ifd, exifparser *exifdata, ExifData *tag_data) {
   int numtags,i, tag;
-  char *ifdp, *data;
+  unsigned char *ifdp, *data;
 
   /* Sanity check first: */
   exif_debug=1;
@@ -396,7 +397,7 @@ static int gpi_exif_get_field( int tag_number, int ifd, exifparser *exifdata, Ex
 /*
  * Gets the comment field if it exists.
  */
-static int gpi_exif_get_comment(exifparser *exifdat, char *comment) {
+static int gpi_exif_get_comment(exifparser *exifdat, char **comment) {
   ExifData tagdat;
 
   if (exif_parse_data(exifdat)<0) return 0;
@@ -406,7 +407,8 @@ static int gpi_exif_get_comment(exifparser *exifdat, char *comment) {
     if (exif_debug) printf("No comment field in this image\n");
     return 0;
   } else {
-    comment = tagdat.data;
+    *comment = malloc (tagdat.size+1);
+    memcpy(*comment,tagdat.data,tagdat.size);
     return tagdat.size;
   }  
 }
@@ -421,7 +423,7 @@ unsigned char *gpi_exif_get_thumbnail_and_size(exifparser *exifdat, long *size)
   unsigned char *imagedata,*exifimg,*newimg,*curptr;
   unsigned int entry;
   long dataptr,dsize,tag,datvec,tmp;
-  int i,j;
+  unsigned int i,j;
 
   exif_debug=1;
   if (exif_parse_data(exifdat)<0) return(NULL);
@@ -444,8 +446,11 @@ unsigned char *gpi_exif_get_thumbnail_and_size(exifparser *exifdat, long *size)
       printf("Decoding EXIF fields in thumbnail\n");
       gpi_exif_get_field( EXIF_Model, -1, exifdat, &owner);
       printf("Camera model: %s\n",owner.data);
-      printf("Comment for this picture (%d chars)",gpi_exif_get_comment( exifdat, comment));
-      /*printf(" -> %s\n",comment); Not OK on Linux */
+      printf("Comment for this picture (%d chars)",gpi_exif_get_comment( exifdat, &comment));
+      if (comment) { 
+          printf(" -> %s\n",comment);
+          free(comment);
+      }
       gpi_exif_get_field( EXIF_SubjectDistance, 2, exifdat, &owner);
       /*      dump_exif(exifdat);       */
   }
@@ -456,6 +461,7 @@ unsigned char *gpi_exif_get_thumbnail_and_size(exifparser *exifdat, long *size)
       fprintf(stderr,"Too few ifds, doesn't look right. Giving up\n");
     }
     *size = 0;
+    free(newimg);
     return(NULL); /* Things don't look right...*/
   }
 
@@ -479,6 +485,7 @@ unsigned char *gpi_exif_get_thumbnail_and_size(exifparser *exifdat, long *size)
     if (dsize==-1){
       fprintf(stderr,"No Jpeg size tag for thumbnail, skipping\n");
       *size = 0;
+      free(newimg);
       return(NULL);
     }
     imagedata=exifdat->data+tmp;
@@ -492,6 +499,7 @@ unsigned char *gpi_exif_get_thumbnail_and_size(exifparser *exifdat, long *size)
   if (tmp==-1) {
     fprintf(stderr,"gpe_get_thumbnail: Tiff or jpeg data not found, skipping\n");
     *size = 0;
+    free(newimg);
     return(NULL);
   }
   imagedata=exifdat->data+tmp;
@@ -500,6 +508,7 @@ unsigned char *gpi_exif_get_thumbnail_and_size(exifparser *exifdat, long *size)
   if (dataptr==-1) {
     printf("Split two\n");
     *size = 0;
+    free(newimg);
     return(NULL);
   }
 
@@ -552,7 +561,7 @@ unsigned char *gpi_exif_get_thumbnail_and_size(exifparser *exifdat, long *size)
  *
  * returns: tag ID number
  */
-static int gpi_tagnum( char *data,int tagind){
+static int gpi_tagnum( unsigned char *data,int tagind){
  return(gpi_exif_get_lilend(data+tagind*12+2,2));
 }
 
