@@ -17,8 +17,8 @@
  * \par
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 #include "config.h"
 #include <gphoto2/gphoto2-widget.h>
@@ -29,8 +29,6 @@
 #include <gphoto2/gphoto2-result.h>
 #include <gphoto2/gphoto2-port-log.h>
 
-#define CHECK_NULL(r)        {if (!(r)) return (GP_ERROR_BAD_PARAMETERS);}
-
 /**
  * CameraWidget:
  *
@@ -51,7 +49,7 @@ struct _CameraWidget {
 	float   value_float;
 
 	/* For Radio and Menu */
-	char    choice [100] [64];
+	char    **choice;
 	int     choice_count;
 
 	/* For Range */
@@ -60,12 +58,15 @@ struct _CameraWidget {
 	float   increment;
 
 	/* Child info */
-	CameraWidget *children [64];
+	CameraWidget **children;
 	int           children_count;
 
 	/* Widget was changed */
 	int     changed;
 
+	/* Widget is read only */
+	int	readonly;
+
 	/* Reference count */
 	int     ref_count;
 
@@ -92,13 +93,11 @@ int
 gp_widget_new (CameraWidgetType type, const char *label, 
 		   CameraWidget **widget) 
 {
-	int x;
 	static int i = 0;
 
-	CHECK_NULL (label && widget);
+	C_PARAMS (label && widget);
 
-	*widget = (CameraWidget*) malloc (sizeof (CameraWidget));
-	memset (*widget, 0, sizeof (CameraWidget));
+	C_MEM (*widget = calloc (1, sizeof (CameraWidget)));
 
 	(*widget)->type = type;
 	strcpy ((*widget)->label, label);
@@ -110,16 +109,15 @@ gp_widget_new (CameraWidgetType type, const char *label,
 
         (*widget)->ref_count    	= 1;
 	(*widget)->choice_count 	= 0;
+	(*widget)->choice 		= NULL;
+	(*widget)->readonly 		= 0;
 	(*widget)->id			= i++;
 
         /* Clear all children pointers */
-	memset ((*widget)->children, 0, sizeof((*widget)->children));
+	free ((*widget)->children);
+	(*widget)->children = NULL;
 	(*widget)->children_count = 0;
 
-	/* Clear out the choices */
-	for (x = 0; x < sizeof((*widget)->choice)/sizeof((*widget)->choice[0]); x++)
-		strcpy ((*widget)->choice[x], "");
-
 	return (GP_OK);
 }
 
@@ -133,21 +131,21 @@ gp_widget_new (CameraWidgetType type, const char *label,
 int
 gp_widget_free (CameraWidget *widget)
 {
-	CHECK_NULL (widget);
+	int x;
+	C_PARAMS (widget);
 
 	/* Free children recursively */
 	if ((widget->type == GP_WIDGET_WINDOW) ||
 	    (widget->type == GP_WIDGET_SECTION)) {
-	    	int x;
-
 	    	for (x = 0; x < gp_widget_count_children (widget); x++)
 			gp_widget_free (widget->children[x]);
+		free (widget->children);
 	}
-	    	
-        if (widget->value_string)
-		free (widget->value_string);
+	for (x = 0; x < widget->choice_count; x++)
+		free (widget->choice[x]);
+	free (widget->choice);
+	free (widget->value_string);
 	free (widget);
-
 	return (GP_OK);
 }
 
@@ -161,7 +159,7 @@ gp_widget_free (CameraWidget *widget)
 int
 gp_widget_ref (CameraWidget *widget) 
 {
-	CHECK_NULL (widget);
+	C_PARAMS (widget);
 
 	widget->ref_count += 1;
 
@@ -178,7 +176,7 @@ gp_widget_ref (CameraWidget *widget)
 int
 gp_widget_unref (CameraWidget *widget) 
 {
-	CHECK_NULL (widget);
+	C_PARAMS (widget);
 
 	widget->ref_count -= 1;
 
@@ -199,7 +197,7 @@ gp_widget_unref (CameraWidget *widget)
 int
 gp_widget_get_info (CameraWidget *widget, const char **info)
 {
-	CHECK_NULL (widget && info);
+	C_PARAMS (widget && info);
 
 	*info = widget->info;
 	return (GP_OK);
@@ -217,7 +215,7 @@ gp_widget_get_info (CameraWidget *widget, const char **info)
 int
 gp_widget_set_info (CameraWidget *widget, const char *info)
 {
-	CHECK_NULL (widget && info);
+	C_PARAMS (widget && info);
 
 	strcpy (widget->info, info);
 	return (GP_OK);
@@ -234,7 +232,7 @@ gp_widget_set_info (CameraWidget *widget, const char *info)
 int
 gp_widget_get_name (CameraWidget *widget, const char **name)
 {
-	CHECK_NULL (widget && name);
+	C_PARAMS (widget && name);
 
 	*name = widget->name;
 	return (GP_OK);
@@ -251,7 +249,7 @@ gp_widget_get_name (CameraWidget *widget, const char **name)
 int
 gp_widget_set_name (CameraWidget *widget, const char *name)
 {
-	CHECK_NULL (widget && name);
+	C_PARAMS (widget && name);
 
 	strcpy (widget->name, name);
 	return (GP_OK);
@@ -270,7 +268,7 @@ gp_widget_set_name (CameraWidget *widget, const char *name)
 int
 gp_widget_get_id (CameraWidget *widget, int *id)
 {
-	CHECK_NULL (widget && id);
+	C_PARAMS (widget && id);
 
 	*id = widget->id;
 	return (GP_OK);
@@ -290,12 +288,50 @@ gp_widget_get_id (CameraWidget *widget, int *id)
 int
 gp_widget_set_changed (CameraWidget *widget, int changed)
 {
-	CHECK_NULL (widget);
+	C_PARAMS (widget);
 
 	widget->changed = changed;
 	return (GP_OK);
 }
 
+/**
+ * \brief Tells that the widget is readonly
+ *
+ * @param widget a #CameraWidget
+ * @param changed a boolean whether we are readonly or not
+ * @return a gphoto2 error code
+ *
+ * Sets the readonly of the CameraWidget depending on 
+ * the changed parameter.
+ *
+ * Only useful when called from the camera driver.
+ **/
+int
+gp_widget_set_readonly (CameraWidget *widget, int readonly)
+{
+	C_PARAMS (widget);
+
+	widget->readonly = readonly;
+	return (GP_OK);
+}
+
+/**
+ * \brief Retrieves the readonly state of the #CameraWidget
+ *
+ * @param widget a #CameraWidget
+ * @param readonly
+ * @return a gphoto2 error code.
+ *
+ **/
+int
+gp_widget_get_readonly (CameraWidget *widget, int *readonly) 
+{
+	C_PARAMS (widget && readonly);
+
+	*readonly = widget->readonly;
+	return (GP_OK);
+}
+
 /**
  * \brief Retrieves the type of the #CameraWidget
  *
@@ -307,7 +343,7 @@ gp_widget_set_changed (CameraWidget *widget, int changed)
 int
 gp_widget_get_type (CameraWidget *widget, CameraWidgetType *type) 
 {
-	CHECK_NULL (widget && type);
+	C_PARAMS (widget && type);
 
 	*type = widget->type;
 	return (GP_OK);
@@ -324,7 +360,7 @@ gp_widget_get_type (CameraWidget *widget, CameraWidgetType *type)
 int
 gp_widget_get_label (CameraWidget *widget, const char **label) 
 {
-	CHECK_NULL (widget && label);
+	C_PARAMS (widget && label);
 
 	*label = widget->label;
 	return (GP_OK);
@@ -338,16 +374,16 @@ gp_widget_get_label (CameraWidget *widget, const char **label)
  * @return a gphoto2 error code.
  *
  * Please pass
- * (char*) for GP_WIDGET_MENU, GP_WIDGET_TEXT,
+ * (char*) for GP_WIDGET_MENU, GP_WIDGET_TEXT, GP_WIDGET_RADIO,
  * (float) for GP_WIDGET_RANGE,
- * (int)   for GP_WIDGET_DATE, GP_WIDGET_TOGGLE, GP_WIDGET_RADIO, and
+ * (int)   for GP_WIDGET_DATE, GP_WIDGET_TOGGLE, and
  * (CameraWidgetCallback) for GP_WIDGET_BUTTON.
  *
  **/
 int
 gp_widget_set_value (CameraWidget *widget, const void *value) 
 {
-	CHECK_NULL (widget && value);
+	C_PARAMS (widget && value);
 
         switch (widget->type) {
 	case GP_WIDGET_BUTTON:
@@ -356,9 +392,9 @@ gp_widget_set_value (CameraWidget *widget, const void *value)
 	case GP_WIDGET_MENU:
 	case GP_WIDGET_RADIO:
         case GP_WIDGET_TEXT:
-		gp_log (GP_LOG_DEBUG, "gphoto2-widget", "Setting value to "
-			"'%s'...", (char*) value);
-	        if (widget->value_string) {
+		GP_LOG_D ("Setting value of widget '%s' to '%s'...",
+			widget->label, (char*) value);
+		if (widget->value_string) {
                 	if (strcmp (widget->value_string, (char*) value))
                     		widget->changed = 1;
                 	free (widget->value_string);
@@ -397,7 +433,7 @@ gp_widget_set_value (CameraWidget *widget, const void *value)
 int
 gp_widget_get_value (CameraWidget *widget, void *value) 
 {
-	CHECK_NULL (widget && value);
+	C_PARAMS (widget && value);
 
         switch (widget->type) {
 	case GP_WIDGET_BUTTON:
@@ -433,13 +469,13 @@ gp_widget_get_value (CameraWidget *widget, void *value)
 int
 gp_widget_append (CameraWidget *widget, CameraWidget *child) 
 {
-	CHECK_NULL (widget && child);
+	C_PARAMS (widget && child);
 
 	/* Return if they can't have any children */
-        if ((widget->type != GP_WIDGET_WINDOW) && 
-	    (widget->type != GP_WIDGET_SECTION))
-		return (GP_ERROR_BAD_PARAMETERS);
+        C_PARAMS ((widget->type == GP_WIDGET_WINDOW) ||
+                  (widget->type == GP_WIDGET_SECTION));
 
+	C_MEM (widget->children = realloc(widget->children, sizeof(CameraWidget*)*(widget->children_count+1)));
 	widget->children[widget->children_count] = child;
 	widget->children_count += 1;
 	child->parent = widget;
@@ -461,12 +497,13 @@ gp_widget_prepend (CameraWidget *widget, CameraWidget *child)
 {
 	int x;
 
-	CHECK_NULL (widget && child);
+	C_PARAMS (widget && child);
 
 	/* Return if they can't have any children */
-	if ((widget->type != GP_WIDGET_WINDOW) && 
-	    (widget->type != GP_WIDGET_SECTION))
-		return (GP_ERROR_BAD_PARAMETERS);
+	C_PARAMS ((widget->type == GP_WIDGET_WINDOW) ||
+		  (widget->type == GP_WIDGET_SECTION));
+
+	C_MEM (widget->children = realloc(widget->children, sizeof(CameraWidget*)*(widget->children_count+1)));
 
 	/* Shift down 1 */
 	for (x = widget->children_count; x > 0; x--)
@@ -491,7 +528,7 @@ gp_widget_prepend (CameraWidget *widget, CameraWidget *child)
 int
 gp_widget_count_children (CameraWidget *widget) 
 {
-	CHECK_NULL (widget);
+	C_PARAMS (widget);
 
 	return (widget->children_count);
 }
@@ -509,10 +546,8 @@ int
 gp_widget_get_child (CameraWidget *widget, int child_number, 
 		     CameraWidget **child) 
 {
-	CHECK_NULL (widget && child);
-
-	if (child_number >= widget->children_count)
-		return (GP_ERROR_BAD_PARAMETERS);
+	C_PARAMS (widget && child);
+	C_PARAMS (child_number < widget->children_count);
 
 	*child = widget->children[child_number];
 	return (GP_OK);
@@ -533,7 +568,7 @@ gp_widget_get_child_by_label (CameraWidget *widget, const char *label,
 {
 	int x;
 
-	CHECK_NULL (widget && label && child);
+	C_PARAMS (widget && label && child);
 
 	if (strcmp (widget->label, label) == 0) {
 		*child = widget;
@@ -569,7 +604,7 @@ gp_widget_get_child_by_id (CameraWidget *widget, int id, CameraWidget **child)
 {
 	int x;
 
-	CHECK_NULL (widget && child);
+	C_PARAMS (widget && child);
 
 	if (widget->id == id) {
 		*child = widget;
@@ -605,7 +640,7 @@ gp_widget_get_child_by_name (CameraWidget *widget, const char *name,
 {
 	int x;
 
-	CHECK_NULL (widget && child);
+	C_PARAMS (widget && child);
 
 	if (!strcmp (widget->name, name)) {
 		*child = widget;
@@ -638,7 +673,7 @@ gp_widget_get_child_by_name (CameraWidget *widget, const char *name,
 int
 gp_widget_get_parent (CameraWidget *widget, CameraWidget **parent)
 {
-	CHECK_NULL (widget && parent);
+	C_PARAMS (widget && parent);
 
 	*parent = widget->parent;
 
@@ -656,7 +691,7 @@ gp_widget_get_parent (CameraWidget *widget, CameraWidget **parent)
 int
 gp_widget_get_root (CameraWidget *widget, CameraWidget **root) 
 {
-	CHECK_NULL (widget && root);
+	C_PARAMS (widget && root);
 
 	if (widget->parent) 
 		return (gp_widget_get_root (widget->parent, root));
@@ -679,10 +714,8 @@ gp_widget_get_root (CameraWidget *widget, CameraWidget **root)
 int
 gp_widget_set_range (CameraWidget *range, float min, float max, float increment)
 {
-	CHECK_NULL (range);
-	
-	if (range->type != GP_WIDGET_RANGE)
-		return (GP_ERROR_BAD_PARAMETERS);
+	C_PARAMS (range);
+	C_PARAMS (range->type == GP_WIDGET_RANGE);
 
 	range->min = min;
 	range->max = max;
@@ -705,9 +738,8 @@ int
 gp_widget_get_range (CameraWidget *range, float *min, float *max, 
 		     float *increment) 
 {
-	CHECK_NULL (range && min && max && increment);
-	if (range->type != GP_WIDGET_RANGE)
-		return (GP_ERROR_BAD_PARAMETERS);
+	C_PARAMS (range && min && max && increment);
+	C_PARAMS (range->type == GP_WIDGET_RANGE);
 
 	*min = range->min;
 	*max = range->max;
@@ -727,18 +759,13 @@ gp_widget_get_range (CameraWidget *range, float *min, float *max,
 int
 gp_widget_add_choice (CameraWidget *widget, const char *choice) 
 {
-	CHECK_NULL (widget && choice);
-	if ((widget->type != GP_WIDGET_RADIO) &&
-	    (widget->type != GP_WIDGET_MENU))
-		return (GP_ERROR_BAD_PARAMETERS);
+	C_PARAMS (widget && choice);
+	C_PARAMS ((widget->type == GP_WIDGET_RADIO) ||
+		  (widget->type == GP_WIDGET_MENU));
 
-	/* static list of choices is full */
-	if (widget->choice_count >= sizeof(widget->choice)/sizeof(widget->choice[0]))
-		return (GP_ERROR);
-
-	strncpy (widget->choice[widget->choice_count], choice, sizeof(widget->choice[0]));
+	C_MEM (widget->choice = realloc (widget->choice, sizeof(char*)*(widget->choice_count+1)));
+	widget->choice[widget->choice_count] = strdup(choice);
 	widget->choice_count += 1;
-
 	return (GP_OK);
 }
 
@@ -752,10 +779,9 @@ gp_widget_add_choice (CameraWidget *widget, const char *choice)
 int
 gp_widget_count_choices (CameraWidget *widget) 
 {
-	CHECK_NULL (widget);
-	if ((widget->type != GP_WIDGET_RADIO) &&
-	    (widget->type != GP_WIDGET_MENU))
-		return (GP_ERROR_BAD_PARAMETERS);
+	C_PARAMS (widget);
+	C_PARAMS ((widget->type == GP_WIDGET_RADIO) ||
+		  (widget->type == GP_WIDGET_MENU));
 
 	return (widget->choice_count);
 }
@@ -773,13 +799,10 @@ int
 gp_widget_get_choice (CameraWidget *widget, int choice_number, 
 		      const char **choice) 
 {
-	CHECK_NULL (widget && choice);
-	if ((widget->type != GP_WIDGET_RADIO) &&
-	    (widget->type != GP_WIDGET_MENU))
-		return (GP_ERROR_BAD_PARAMETERS);
-
-	if (choice_number >= widget->choice_count)
-		return (GP_ERROR_BAD_PARAMETERS);
+	C_PARAMS (widget && choice);
+	C_PARAMS ((widget->type == GP_WIDGET_RADIO) ||
+		  (widget->type == GP_WIDGET_MENU));
+	C_PARAMS (choice_number < widget->choice_count);
 
 	*choice = widget->choice[choice_number];
 	return (GP_OK);
@@ -792,19 +815,14 @@ gp_widget_get_choice (CameraWidget *widget, int choice_number,
  * @return a gphoto2 error code or changed flag.
  *
  * Returns 1 if the state of the #CameraWidget has been changed or 0 if not.
- * In addition, it resets the changed flag to 0.
+ *
+ * Note: this formerly cleared the changed state. It no longer does with 2.5.11.
  *
  **/
 int
 gp_widget_changed (CameraWidget *widget) 
 {
-        int val;
+	C_PARAMS (widget);
 
-	CHECK_NULL (widget);
-
-        val = widget->changed;
-        widget->changed = 0;
-
-        return (val);
+        return widget->changed;
 }
-
