@@ -24,13 +24,9 @@
 #include <gphoto2/gphoto2-port-library.h>
 
 #include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 #include <fcntl.h>
 #include <errno.h>
-#include <sys/time.h>
-#include <sys/param.h>
-#include <dirent.h>
 #include <string.h>
 
 #include <libusb.h>
@@ -66,8 +62,7 @@
 	}\
 } while(0)
 
-#ifndef HAVE_LIBUSB_STRERROR
-static const char *libusb_strerror(int r)
+static const char *my_libusb_strerror(int r)
 {
 	switch (r) {
 	case LIBUSB_SUCCESS:			return "Success";
@@ -88,12 +83,10 @@ static const char *libusb_strerror(int r)
 	}
 }
 
-#endif
-
 static int log_on_libusb_error_helper( int _r, const char* _func, const char* file, int line, const char* func ) {
 	if (_r < LIBUSB_SUCCESS)
 		gp_log_with_source_location(GP_LOG_ERROR, file, line, func,
-					    "'%s' failed: %s (%d)", _func, libusb_strerror(_r), _r);
+					    "'%s' failed: %s (%d)", _func, my_libusb_strerror(_r), _r);
 	return _r;
 }
 #define LOG_ON_LIBUSB_E( RESULT ) log_on_libusb_error_helper( (RESULT), #RESULT, __FILE__, __LINE__, __func__ )
@@ -143,6 +136,11 @@ struct _GPPortPrivateLibrary {
 	struct libusb_device_descriptor	*descs;
 	libusb_device			**devs;
 
+#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
+	/* for dumping the read usb content */
+	int				logfd;
+#endif
+
 #define INTERRUPT_TRANSFERS 10
 	struct libusb_transfer		*transfers[INTERRUPT_TRANSFERS];
 	int 				nrofirqs;
@@ -174,7 +172,7 @@ load_devicelist (GPPortPrivateLibrary *pl) {
 		int 	i;
 
 		pl->nrofdevs = libusb_get_device_list (pl->ctx, &pl->devs);
-		C_MEM (pl->descs = malloc (sizeof(pl->descs[0])*pl->nrofdevs));
+		C_MEM (pl->descs = calloc (pl->nrofdevs, sizeof(pl->descs[0])));
 		for (i=0;i<pl->nrofdevs;i++)
 			LOG_ON_LIBUSB_E (libusb_get_device_descriptor(pl->devs[i], &pl->descs[i]));
 	}
@@ -202,10 +200,10 @@ gp_port_library_list (GPPortInfoList *list)
 	gp_port_info_set_type (info, GP_PORT_USB);
 	gp_port_info_set_name (info, "");
 	gp_port_info_set_path (info, "^usb:");
-	C_GP (gp_port_info_list_append (list, info));
+	gp_port_info_list_append (list, info); /* do not check return value, it might be -1 */
 
 	nrofdevs = libusb_get_device_list (ctx, &devs);
-	C_MEM (descs = malloc (sizeof(descs[0])*nrofdevs));
+	C_MEM (descs = calloc (nrofdevs, sizeof(descs[0])));
 	for (i=0;i<nrofdevs;i++)
 		LOG_ON_LIBUSB_E (libusb_get_device_descriptor(devs[i], &descs[i]));
 
@@ -310,6 +308,11 @@ gp_port_library_list (GPPortInfoList *list)
 		gp_port_info_set_path (info, path);
 		C_GP (gp_port_info_list_append (list, info));
 	}
+
+	libusb_free_device_list (devs, 1);
+	libusb_exit (ctx); /* should free all stuff above */
+	free (descs);
+
 	/* This will only be added if no other device was ever added.
 	 * Users doing "usb:" usage will enter the regular expression matcher case. */
 	if (nrofdevices == 0) {
@@ -319,9 +322,6 @@ gp_port_library_list (GPPortInfoList *list)
 		gp_port_info_set_path (info, "usb:");
 		C_GP (gp_port_info_list_append (list, info));
 	}
-	libusb_free_device_list (devs, 1);
-	libusb_exit (ctx); /* should free all stuff above */
-	free (descs);
 	return (GP_OK);
 }
 
@@ -337,6 +337,10 @@ static int gp_libusb1_init (GPPort *port)
 		port->pl = NULL;
 		return GP_ERROR_IO;
 	}
+#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
+	unlink("usblog.raw");
+	port->pl->logfd = open("usblog.raw",O_CREAT|O_WRONLY,0644);
+#endif
 #if 0
 	libusb_set_debug (port->pl->ctx, 255);
 #endif
@@ -348,6 +352,9 @@ gp_libusb1_exit (GPPort *port)
 {
 	if (port->pl) {
 		free (port->pl->descs);
+#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
+		if (port->pl->logfd >=0 ) close (port->pl->logfd);
+#endif
 		if (port->pl->nrofdevs)
 			libusb_free_device_list (port->pl->devs, 1);
 		libusb_exit (port->pl->ctx);
@@ -432,7 +439,7 @@ gp_libusb1_open (GPPort *port)
 }
 
 static int
-gp_libusb1_close (GPPort *port)
+_close_async_interrupts(GPPort *port)
 {
 	int i, haveone;
 	struct timeval tv;
@@ -452,11 +459,14 @@ gp_libusb1_close (GPPort *port)
 			GP_LOG_D("canceling transfer %d:%p (status %d)",i, port->pl->transfers[i], port->pl->transfers[i]->status);
 			/* this happens if the transfer is completed for instance, but not reaped. we cannot cancel it. */
 			if (LOG_ON_LIBUSB_E(libusb_cancel_transfer(port->pl->transfers[i])) < 0) {
-				libusb_free_transfer (port->pl->transfers[i]);
+				/* do not libusb_free_transfer (port->pl->transfers[i]); causes crashes */
 				port->pl->transfers[i] = NULL;
 			}
 		}
 	}
+	tv.tv_sec = 0;
+	tv.tv_usec = 0;
+	LOG_ON_LIBUSB_E (libusb_handle_events_timeout(port->pl->ctx, &tv));
 	/* Do just one round ... this should be sufficient and avoids endless loops. */
 	haveone = 0;
 	for (i = 0; i < sizeof(port->pl->transfers)/sizeof(port->pl->transfers[0]); i++) {
@@ -467,7 +477,18 @@ gp_libusb1_close (GPPort *port)
 	}
 	if (haveone)
 		LOG_ON_LIBUSB_E (libusb_handle_events(port->pl->ctx));
+	return GP_OK;
+}
 
+static int
+gp_libusb1_close (GPPort *port)
+{
+	C_PARAMS (port);
+
+	if (port->pl->dh == NULL)
+		return GP_OK;
+
+	_close_async_interrupts(port);
 
 	if (libusb_release_interface (port->pl->dh,
 				   port->settings.usb.interface) < 0) {
@@ -501,7 +522,10 @@ gp_libusb1_close (GPPort *port)
 	libusb_close (port->pl->dh);
 
 	free (port->pl->irqs);
+	port->pl->irqs = NULL;
 	free (port->pl->irqlens);
+	port->pl->irqlens = NULL;
+	port->pl->nrofirqs = 0;
 	port->pl->dh = NULL;
 	return GP_OK;
 }
@@ -558,6 +582,9 @@ gp_libusb1_read(GPPort *port, char *bytes, int size)
 			(unsigned char*)bytes, size, &curread, port->timeout),
 		  GP_ERROR_IO_READ );
 
+#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
+	write(port->pl->logfd, bytes, curread);
+#endif
         return curread;
 }
 
@@ -566,6 +593,9 @@ gp_libusb1_reset(GPPort *port)
 {
 	C_PARAMS (port && port->pl->dh);
 
+	/* earlier libusb 1 versions get crashes otherwise */
+	_close_async_interrupts(port);
+
 	C_LIBUSB (libusb_reset_device (port->pl->dh), GP_ERROR_IO);
 
 	return GP_OK;
