@@ -2,8 +2,8 @@ var NAVTREE =
 [
   [ "libgphoto2 photo camera library (libgphoto2) API", "index.html", [
     [ "Deprecated List", "deprecated.html", null ],
-    [ "Data Structures", null, [
-      [ "Data Structures", "annotated.html", "annotated" ],
+    [ "Data Structures", "annotated.html", [
+      [ "Data Structures", "annotated.html", "annotated_dup" ],
       [ "Data Fields", "functions.html", [
         [ "All", "functions.html", null ],
         [ "Variables", "functions_vars.html", null ]
@@ -28,11 +28,11 @@ var NAVTREEINDEX =
 [
 "__stdint_8h_source.html",
 "gphoto2-camera_8c.html",
-"gphoto2-file_8h.html#aca2b7ed7d1b91fcdb0578ee71142a101",
-"gphoto2-port-info-list_8h.html#af1f1553e664584f74724a8825829bc09",
-"gphoto2-version_8c.html#a039629a84ef4c735b1124901438a7468",
-"struct__CameraFilesystem.html#a48290da7bdd03d2a9f24a4bf6fa840dd",
-"union__GPPortSettings.html#ac5d97643d4d2c83edd17731949966b49"
+"gphoto2-file_8h.html#ab9f339bdf374343272c62d2352753d69a5f9e4fc5c19d2f49b1539d61ebc7d824",
+"gphoto2-port-info-list_8h.html#a70f65ef80a59174adfa52c3e199752d4",
+"gphoto2-setting_8c.html",
+"struct__CameraFileInfoPreview.html#a12d6115703a502a24cd5b193b75d666a",
+"template_8c.html#a9127c82b044f56fac4cbceb92b9e0217"
 ];
 
 var SYNCONMSG = 'click to disable panel synchronisation';
