@@ -1,5 +1,15 @@
 var NAVTREEINDEX4 =
 {
+"gphoto2-setting_8c.html":[2,0,2,17],
+"gphoto2-setting_8c.html#a1061828e4e2cafb14086ac2bed47f80d":[2,0,2,17,2],
+"gphoto2-setting_8c.html#a4b50249c04f6a48156bad839827e67b7":[2,0,2,17,3],
+"gphoto2-setting_8c.html#a6100f1cf5f3e90c2491807b6db1796af":[2,0,2,17,4],
+"gphoto2-setting_8c.html#a8fb447618db946a9e2a596d9ea18763f":[2,0,2,17,1],
+"gphoto2-setting_8h.html":[2,0,1,8],
+"gphoto2-setting_8h.html#a4b50249c04f6a48156bad839827e67b7":[2,0,1,8,0],
+"gphoto2-setting_8h.html#a6100f1cf5f3e90c2491807b6db1796af":[2,0,1,8,1],
+"gphoto2-setting_8h_source.html":[2,0,1,8],
+"gphoto2-version_8c.html":[2,0,2,18],
 "gphoto2-version_8c.html#a039629a84ef4c735b1124901438a7468":[2,0,2,18,0],
 "gphoto2-version_8h.html":[2,0,1,9],
 "gphoto2-version_8h.html#a039629a84ef4c735b1124901438a7468":[2,0,1,9,0],
@@ -41,9 +51,9 @@ var NAVTREEINDEX4 =
 "gphoto2-widget_8h.html#a160a340c58e006a1a4de0ac77e02e136":[2,0,1,10,17],
 "gphoto2-widget_8h.html#a20783a5aec3e3a7cd4c8ce3af7a92361":[2,0,1,10,18],
 "gphoto2-widget_8h.html#a3017f318e2dadaf82d5d55140594bcdf":[2,0,1,10,11],
-"gphoto2-widget_8h.html#a43017455ef825f78ea479b8d3facb87f":[2,0,1,10,1],
 "gphoto2-widget_8h.html#a467afee62da7ca075c8b24c71cfd812a":[2,0,1,10,9],
 "gphoto2-widget_8h.html#a4c4ea20c4d069c418399754822644174":[2,0,1,10,27],
+"gphoto2-widget_8h.html#a4ff53441fd74eb630a997d461220f781":[2,0,1,10,1],
 "gphoto2-widget_8h.html#a582ecd400d10bb040b35c317c48cf638":[2,0,1,10,29],
 "gphoto2-widget_8h.html#a58d57830ff9a7ce66b1b4b9faa93948a":[2,0,1,10,31],
 "gphoto2-widget_8h.html#a5b80fa207622d0efc676c3219c6150eb":[2,0,1,10,25],
@@ -202,6 +212,7 @@ var NAVTREEINDEX4 =
 "struct__Camera.html#af55d6e519c6b49eb11e518c419b1627f":[1,0,0,1],
 "struct__CameraAbilitiesList.html":[1,0,1],
 "struct__CameraAbilitiesList.html#a3a18bfd44cf3ae22ebea7587153debb9":[1,0,1,0],
+"struct__CameraAbilitiesList.html#a8293a0fc888d18b6f3c6ee5a424fb884":[1,0,1,2],
 "struct__CameraAbilitiesList.html#ad57e34bd38b93bcdaf2b47629105531d":[1,0,1,1],
 "struct__CameraFile.html":[1,0,2],
 "struct__CameraFile.html#a05fa8fdab784c185cabe2485c7820875":[1,0,2,10],
@@ -216,9 +227,9 @@ var NAVTREEINDEX4 =
 "struct__CameraFile.html#ae2356552d82970aa6dc283c3d6376ec3":[1,0,2,0],
 "struct__CameraFile.html#af4d719b672fb67028a17b8f8fc0a0409":[1,0,2,3],
 "struct__CameraFileHandler.html":[1,0,3],
-"struct__CameraFileHandler.html#a35f8d8fea5283e71c06c9165a03084fc":[1,0,3,0],
-"struct__CameraFileHandler.html#a69173afefd78d998731c1d5318522754":[1,0,3,2],
-"struct__CameraFileHandler.html#ab6648ddca62242768988604017443d54":[1,0,3,1],
+"struct__CameraFileHandler.html#a5bbba5d19117be4affa56639088025cc":[1,0,3,2],
+"struct__CameraFileHandler.html#aaa08112442b99b25e1c230f61a72decc":[1,0,3,1],
+"struct__CameraFileHandler.html#ad2e1e6423213516de8194fcfd3ef4d0a":[1,0,3,0],
 "struct__CameraFileInfo.html":[1,0,4],
 "struct__CameraFileInfo.html#a1179620523bf1e56b47bcc7050b2cd76":[1,0,4,2],
 "struct__CameraFileInfo.html#a37f70742bfcbfae4e06e6bfcc235a772":[1,0,4,0],
@@ -238,16 +249,5 @@ var NAVTREEINDEX4 =
 "struct__CameraFileInfoFile.html#ae7acce7dab662a7dfb088602176ac20b":[1,0,6,2],
 "struct__CameraFileInfoFile.html#af076fe6da9612a0167acf489c40f31e8":[1,0,6,1],
 "struct__CameraFileInfoPreview.html":[1,0,7],
-"struct__CameraFileInfoPreview.html#a08eb4d953d2c9f4f7e8ce54223bb8ea9":[1,0,7,3],
-"struct__CameraFileInfoPreview.html#a12d6115703a502a24cd5b193b75d666a":[1,0,7,5],
-"struct__CameraFileInfoPreview.html#a2cc5f80f66aa90d7c276df9ecd530490":[1,0,7,1],
-"struct__CameraFileInfoPreview.html#a42e378c8d0668052b39c0cca22382dd7":[1,0,7,4],
-"struct__CameraFileInfoPreview.html#a872f4d1625b0008c625138f33ef3e4cb":[1,0,7,0],
-"struct__CameraFileInfoPreview.html#a9a3b70779dc8cea4d520b9349bee6ba4":[1,0,7,2],
-"struct__CameraFilesystem.html":[1,0,8],
-"struct__CameraFilesystem.html#a1df04156f0ffe145f2e9d99eea9641ee":[1,0,8,13],
-"struct__CameraFilesystem.html#a2f676eede09ea213a0ef7e8711d6ddf3":[1,0,8,5],
-"struct__CameraFilesystem.html#a31e9fa7b813074658672a512b683a334":[1,0,8,1],
-"struct__CameraFilesystem.html#a32c4b277105e1a86ade9700dbc1cf891":[1,0,8,4],
-"struct__CameraFilesystem.html#a38fcb7940178fa609cd62b8aa9ea5c29":[1,0,8,10]
+"struct__CameraFileInfoPreview.html#a08eb4d953d2c9f4f7e8ce54223bb8ea9":[1,0,7,3]
 };
