@@ -1,5 +1,16 @@
 var NAVTREEINDEX5 =
 {
+"struct__CameraFileInfoPreview.html#a12d6115703a502a24cd5b193b75d666a":[1,0,7,5],
+"struct__CameraFileInfoPreview.html#a2cc5f80f66aa90d7c276df9ecd530490":[1,0,7,1],
+"struct__CameraFileInfoPreview.html#a42e378c8d0668052b39c0cca22382dd7":[1,0,7,4],
+"struct__CameraFileInfoPreview.html#a872f4d1625b0008c625138f33ef3e4cb":[1,0,7,0],
+"struct__CameraFileInfoPreview.html#a9a3b70779dc8cea4d520b9349bee6ba4":[1,0,7,2],
+"struct__CameraFilesystem.html":[1,0,8],
+"struct__CameraFilesystem.html#a1df04156f0ffe145f2e9d99eea9641ee":[1,0,8,13],
+"struct__CameraFilesystem.html#a2f676eede09ea213a0ef7e8711d6ddf3":[1,0,8,5],
+"struct__CameraFilesystem.html#a31e9fa7b813074658672a512b683a334":[1,0,8,1],
+"struct__CameraFilesystem.html#a32c4b277105e1a86ade9700dbc1cf891":[1,0,8,4],
+"struct__CameraFilesystem.html#a38fcb7940178fa609cd62b8aa9ea5c29":[1,0,8,10],
 "struct__CameraFilesystem.html#a48290da7bdd03d2a9f24a4bf6fa840dd":[1,0,8,12],
 "struct__CameraFilesystem.html#a4f95116fa657186a1cb011004451d503":[1,0,8,6],
 "struct__CameraFilesystem.html#a6a7b0038371ba041f35153ad55e302ba":[1,0,8,16],
@@ -47,26 +58,29 @@ var NAVTREEINDEX5 =
 "struct__CameraFilesystemFuncs.html#aecc8d4ddaebcb1ee047331a594ce6c20":[1,0,11,0],
 "struct__CameraFilesystemFuncs.html#afedc6d1a7e923d67db12c5aa2df101b8":[1,0,11,12],
 "struct__CameraFunctions.html":[1,0,12],
-"struct__CameraFunctions.html#a05e1ec8563ae62acbcda7476379004f0":[1,0,12,12],
-"struct__CameraFunctions.html#a0ed0d7256aa2ed3c82fa390b860486d2":[1,0,12,11],
-"struct__CameraFunctions.html#a115074d29c4356a3f20bdb7f192932d0":[1,0,12,9],
+"struct__CameraFunctions.html#a05e1ec8563ae62acbcda7476379004f0":[1,0,12,14],
+"struct__CameraFunctions.html#a0ed0d7256aa2ed3c82fa390b860486d2":[1,0,12,13],
+"struct__CameraFunctions.html#a0fc999ffd155ec790a698527bf8232c4":[1,0,12,19],
+"struct__CameraFunctions.html#a115074d29c4356a3f20bdb7f192932d0":[1,0,12,11],
+"struct__CameraFunctions.html#a1e83ac8e74419d73229b6dd9e149212a":[1,0,12,5],
 "struct__CameraFunctions.html#a26e1c20ccec7a5983c0c563f5a21e053":[1,0,12,2],
-"struct__CameraFunctions.html#a284268b41f1a849f320058a6cfc7aad4":[1,0,12,6],
-"struct__CameraFunctions.html#a3034c0d26ca790676b7a2e344780f869":[1,0,12,5],
-"struct__CameraFunctions.html#a3333e72219a225bec0d294f3a4c87d18":[1,0,12,19],
-"struct__CameraFunctions.html#a515f9abe8387cd85f0cfce4b6717f305":[1,0,12,15],
-"struct__CameraFunctions.html#a595f135aff3af5099f05d14dbd04802a":[1,0,12,13],
-"struct__CameraFunctions.html#a8a5dce8827fb65c09aba4bb48a95bc62":[1,0,12,7],
+"struct__CameraFunctions.html#a284268b41f1a849f320058a6cfc7aad4":[1,0,12,8],
+"struct__CameraFunctions.html#a3034c0d26ca790676b7a2e344780f869":[1,0,12,7],
+"struct__CameraFunctions.html#a3333e72219a225bec0d294f3a4c87d18":[1,0,12,22],
+"struct__CameraFunctions.html#a515f9abe8387cd85f0cfce4b6717f305":[1,0,12,17],
+"struct__CameraFunctions.html#a595f135aff3af5099f05d14dbd04802a":[1,0,12,15],
+"struct__CameraFunctions.html#a8a5dce8827fb65c09aba4bb48a95bc62":[1,0,12,9],
 "struct__CameraFunctions.html#a97669d9398b238a1cad0104da22d358b":[1,0,12,4],
 "struct__CameraFunctions.html#aa247ba281ffdefea96c0a7c82c4d74d1":[1,0,12,0],
-"struct__CameraFunctions.html#aa3e662eb999c79dd80bebb7333588fb1":[1,0,12,16],
-"struct__CameraFunctions.html#aa42394477d9a4f3c3cb3a8b093722075":[1,0,12,17],
+"struct__CameraFunctions.html#aa3e662eb999c79dd80bebb7333588fb1":[1,0,12,18],
+"struct__CameraFunctions.html#aa42394477d9a4f3c3cb3a8b093722075":[1,0,12,20],
 "struct__CameraFunctions.html#aa70e56a844c2f906211a2d1fd44bb162":[1,0,12,3],
-"struct__CameraFunctions.html#aaa16da1dc894cdca2d10b52f4eec9104":[1,0,12,18],
+"struct__CameraFunctions.html#aaa16da1dc894cdca2d10b52f4eec9104":[1,0,12,21],
 "struct__CameraFunctions.html#aab8784a8dc7b2bd19b844852a76b0242":[1,0,12,1],
-"struct__CameraFunctions.html#aded55ef8e46bf8c840705df1dda1b2a7":[1,0,12,14],
-"struct__CameraFunctions.html#ae993834fa4d16eca83cd069cdea37881":[1,0,12,8],
-"struct__CameraFunctions.html#afc4f6db6b18855216c91aa7239a048e6":[1,0,12,10],
+"struct__CameraFunctions.html#ad2ae9500ae590351db5ee8395e2bd492":[1,0,12,6],
+"struct__CameraFunctions.html#aded55ef8e46bf8c840705df1dda1b2a7":[1,0,12,16],
+"struct__CameraFunctions.html#ae993834fa4d16eca83cd069cdea37881":[1,0,12,10],
+"struct__CameraFunctions.html#afc4f6db6b18855216c91aa7239a048e6":[1,0,12,12],
 "struct__CameraList.html":[1,0,13],
 "struct__CameraList.html#a21dfe1d464296a2ff0f234375134f1fe":[1,0,13,2],
 "struct__CameraList.html#a58aa63bdcd5ede08345c1071baf93468":[1,0,13,1],
@@ -153,30 +167,30 @@ var NAVTREEINDEX5 =
 "struct__GPPortInfoList.html#a8aec99439e10d14c42107df22ef85a91":[1,0,21,1],
 "struct__GPPortInfoList.html#af48f32b03aeff690bd1ee23eb1ad505c":[1,0,21,2],
 "struct__GPPortOperations.html":[1,0,22],
-"struct__GPPortOperations.html#a2cfbbd43232484a42a4dd1fb080d053b":[1,0,22,1],
-"struct__GPPortOperations.html#a33051766a2499f496e86a976c6d9b21c":[1,0,22,7],
-"struct__GPPortOperations.html#a333adda6d19ab438bcb7acc1cd8c0834":[1,0,22,15],
-"struct__GPPortOperations.html#a3a1525b263246c182118bb97bc26d2d2":[1,0,22,8],
-"struct__GPPortOperations.html#a59d8d5449abe72118f852c69eb1b1a49":[1,0,22,2],
-"struct__GPPortOperations.html#a5cdd7d2af27f04c5c8c68d2bd216f8d1":[1,0,22,23],
-"struct__GPPortOperations.html#a61a3d4c659f1622094b9808bf8d641ed":[1,0,22,4],
-"struct__GPPortOperations.html#a6b321c5da351dbe910eceed7be85a355":[1,0,22,18],
-"struct__GPPortOperations.html#a77ff264c1332319f06c5bfca933f2d7b":[1,0,22,6],
-"struct__GPPortOperations.html#a80acfcf8db96c04303cabf3ff6f86703":[1,0,22,19],
-"struct__GPPortOperations.html#a88272ade481b95ccfaa70f448b260969":[1,0,22,0],
-"struct__GPPortOperations.html#a8c09ac20352599af1f4895f5f8def257":[1,0,22,13],
-"struct__GPPortOperations.html#a92ca8aab66fd6a1a3eb1a8a3777d5568":[1,0,22,21],
-"struct__GPPortOperations.html#a9c84543d6e5e69fab728f93c5fc5969b":[1,0,22,10],
-"struct__GPPortOperations.html#aa17650b71b8c338fe50adfa7dc987e15":[1,0,22,5],
-"struct__GPPortOperations.html#aa433083fdb5d7a38c8fa752841816a4d":[1,0,22,9],
-"struct__GPPortOperations.html#aab57d791713078b337dd2e0bf0e19d21":[1,0,22,14],
-"struct__GPPortOperations.html#ab82067768f8f5f113d13cad18d6f3e64":[1,0,22,22],
-"struct__GPPortOperations.html#ac960344afb5f60700a1cf4462d5fa29b":[1,0,22,12],
-"struct__GPPortOperations.html#ac9abb4771ff54ba094ade7d55ab67706":[1,0,22,20],
-"struct__GPPortOperations.html#ad323394086f5aaf82a54cb32f5102da8":[1,0,22,16],
-"struct__GPPortOperations.html#ad4b8e668a700dc2a3afa1dc1ceb634df":[1,0,22,3],
-"struct__GPPortOperations.html#ad6e1e871be54bf4ad355dfbca99ef4ff":[1,0,22,17],
-"struct__GPPortOperations.html#ad7769b95b33db4b8f71d4d462002f277":[1,0,22,11],
+"struct__GPPortOperations.html#a00d8f45bd3960238799acf1d8d4c7f56":[1,0,22,20],
+"struct__GPPortOperations.html#a07407d2133e4574254214f7e74d238cb":[1,0,22,8],
+"struct__GPPortOperations.html#a0b1f52e20380bbf73c6e637123162ada":[1,0,22,16],
+"struct__GPPortOperations.html#a13ac3f849f01563e7e4ca31c98e724a8":[1,0,22,23],
+"struct__GPPortOperations.html#a298b1418b1285bd21448db01b5ae8ca8":[1,0,22,12],
+"struct__GPPortOperations.html#a358da6dd0f58d7a23788c78f0d393ed7":[1,0,22,1],
+"struct__GPPortOperations.html#a4a8aed26ed5c79a60e1a373c79a01237":[1,0,22,10],
+"struct__GPPortOperations.html#a6430583b08b74e4b32fbd93c75e30b56":[1,0,22,6],
+"struct__GPPortOperations.html#a6cd2a421efe358bfe37ade2672c50f35":[1,0,22,21],
+"struct__GPPortOperations.html#a7e1a072431ed8f0ebaea45effaf3c12d":[1,0,22,7],
+"struct__GPPortOperations.html#a85ccf0dd38d3d6bb930a57416c7dfe0a":[1,0,22,19],
+"struct__GPPortOperations.html#a87a3341cf4f3b3aa59bbb8dffdac5b1c":[1,0,22,17],
+"struct__GPPortOperations.html#a9d93c583395ba79d0f35fb087aef5479":[1,0,22,2],
+"struct__GPPortOperations.html#ac49ee680bb019193025a19623844a9a1":[1,0,22,18],
+"struct__GPPortOperations.html#ac563b0c074ee83c787ca48bac4f091b1":[1,0,22,22],
+"struct__GPPortOperations.html#ac60092d2eaeaf89f4b113541a8e77284":[1,0,22,11],
+"struct__GPPortOperations.html#ac78fe99c6c42a19a550fbbe6e4058b62":[1,0,22,9],
+"struct__GPPortOperations.html#ac9d92908cf76a394588e3542e81f8475":[1,0,22,14],
+"struct__GPPortOperations.html#ace55ddbf2cfea001509eee7c9dad15cf":[1,0,22,3],
+"struct__GPPortOperations.html#ace7e38af86996479a5cdf6b9aaa9b79e":[1,0,22,15],
+"struct__GPPortOperations.html#ada44977a3929a4cc25580c96a8b33438":[1,0,22,13],
+"struct__GPPortOperations.html#ae89f090f3a36bb944e4b88c586d61cdd":[1,0,22,5],
+"struct__GPPortOperations.html#aeccb4748ea69eecf07ae1b35572f8c85":[1,0,22,4],
+"struct__GPPortOperations.html#af892eace5eaf0a6dae2676ad8ba342f6":[1,0,22,0],
 "struct__GPPortPrivateCore.html":[1,0,23],
 "struct__GPPortPrivateCore.html#a01acb7d55497723970bf96a9bbe78d3f":[1,0,23,1],
 "struct__GPPortPrivateCore.html#a1fd9de117f1f3519bf325cf2fc77ef8c":[1,0,23,3],
@@ -235,19 +249,5 @@ var NAVTREEINDEX5 =
 "template_8c.html#a6b9e41589d48154298976ee65aa9f9d6":[2,0,0,0,0,9],
 "template_8c.html#a6fb50401c2b5419a38d7cdec8468b6ee":[2,0,0,0,0,18],
 "template_8c.html#a75278405e7f034d2b1af80bfd94675fe":[2,0,0,0,0,1],
-"template_8c.html#a8f4cb0c9cb393986b466b60edc3bcd98":[2,0,0,0,0,19],
-"template_8c.html#a9127c82b044f56fac4cbceb92b9e0217":[2,0,0,0,0,22],
-"template_8c.html#aa2bedbf50a33963dbe0938bc83d542d1":[2,0,0,0,0,11],
-"template_8c.html#aab9c38421fc9b44444b9586c3e117d91":[2,0,0,0,0,8],
-"template_8c.html#ab19683ddf354e8f4d401d79489cdab00":[2,0,0,0,0,13],
-"template_8c.html#abaedcea6348683a300838b493f4254a9":[2,0,0,0,0,17],
-"template_8c.html#abc05a8ae36f010d3b6d1f774a397d9b5":[2,0,0,0,0,15],
-"template_8c.html#ac5604f4c487f848626ed2ac56a7c9339":[2,0,0,0,0,20],
-"template_8c.html#ad6759410a0799b25df88f4297d15441f":[2,0,0,0,0,14],
-"template_8c.html#addfb85248f27b40480209cc9de04c6a0":[2,0,0,0,0,3],
-"template_8c.html#af8be80f9a636a9dff8c497cf3cfe0871":[2,0,0,0,0,10],
-"union__GPPortSettings.html":[1,0,24],
-"union__GPPortSettings.html#a0aefcc2253c254cac9ea3fb15f2a059b":[1,0,24,1],
-"union__GPPortSettings.html#a5e89705555ef3dd1abc7baca312cd253":[1,0,24,3],
-"union__GPPortSettings.html#a9fa1454110af3b6425425de105284c8e":[1,0,24,2]
+"template_8c.html#a8f4cb0c9cb393986b466b60edc3bcd98":[2,0,0,0,0,19]
 };
