@@ -431,7 +431,7 @@ udev_parse_params (const func_params_t *params, void **data)
 		"ENV{ID_USB_INTERFACES}==\"\", IMPORT{builtin}=\"usb_id\"\n"
 		/* shortcut the most common camera driver, ptp class, so we avoid parsing 1000
 		 * more rules . It will be completed in udev_begin_func() */
-		"ENV{ID_USB_INTERFACES}==\"*:060101:*\", ENV{ID_GPHOTO2}=\"1\", ENV{GPHOTO2_DRIVER}=\"PTP\"",
+		"ENV{ID_USB_INTERFACES}==\"*:060101:*\", ENV{ID_GPHOTO2}=\"1\", ENV{GPHOTO2_DRIVER}=\"PTP\", ",
 	};
 	static const char * const usbcam_strings[] = {
 		/* UDEV_PRE_0_98 */
@@ -535,7 +535,7 @@ udev_begin_func (const func_params_t *params, void **data)
 		printf ("# this file is autogenerated, local changes will be LOST on upgrades\n");
 		printf ("%s", pdata->begin_string);
 
-		if (pdata->version == UDEV_136 || pdata->version == UDEV_175) {
+		if (pdata->version == UDEV_136 || pdata->version == UDEV_175 || pdata->version == UDEV_201) {
 			if (pdata->mode != NULL || pdata->owner != NULL || pdata->group != NULL) {
 				if (pdata->mode != NULL) {
 					printf("MODE=\"%s\", ", pdata->mode);
@@ -883,22 +883,14 @@ ddb_delayed_head(const func_params_t *params,
 	printf("device \"%s\" {\n", a->model);
 
 	printf("    device_type");
-	first = 1;
-	gpi_enum_to_string(a->device_type, 
-			   gpi_gphoto_device_type_map,
-			   ddb_list_out_func,
-			   (void *) &first);
-	printf(";\n");
+	printf(" %s;\n", gpi_enum_to_string(a->device_type,
+					    gpi_gphoto_device_type_map);
 
 	printf("    driver \"%s\";\n", camlib_basename);
 
 	printf("    driver_status");
-	first = 1;
-	gpi_enum_to_string(a->status, 
-			   gpi_camera_driver_status_map,
-			   ddb_list_out_func,
-			   (void *) &first);
-	printf(";\n");
+	printf(" %s;\n", gpi_enum_to_string(a->status,
+					    gpi_camera_driver_status_map);
 
 	printf("    operations");
 	first = 1;
@@ -1348,6 +1340,11 @@ html_camera_func (
 		op &= ~GP_OPERATION_CAPTURE_IMAGE;
 		if (op) printf (", ");
 	}
+	if (op & GP_OPERATION_TRIGGER_CAPTURE) {
+		printf ("Trigger Capture");
+		op &= ~GP_OPERATION_TRIGGER_CAPTURE;
+		if (op) printf (", ");
+	}
 	if (op & GP_OPERATION_CAPTURE_PREVIEW) {
 		printf ("Liveview");
 		op &= ~GP_OPERATION_CAPTURE_PREVIEW;
